/******************************************************************************
* The MIT License
*
* Copyright (c) 2016, 2017 Paul Young
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use, copy,
* modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
* BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
* ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*****************************************************************************/

#pragma once

static const int PARSE_BUF_SIZE = 8;

// Messages sent to the dialog
static const int KEYER_MSG = WM_USER;

enum Keyer_Message {
	KEYER_MESSAGE_CHAR,				// Arg is character
	KEYER_MESSAGE_CONNECT,			// Arg is zero
	KEYER_MESSAGE_DISCONNECT,		// Arg is error
	KEYER_MESSAGE_SPEED,			// Arg is speed
	KEYER_MESSAGE_FARNSWORTH,		// Arg is speed
	KEYER_MESSAGE_VOLUME,			// Arg is volume
	KEYER_MESSAGE_FREQUENCY,		// Arg is frequency in 10s of Hz
	KEYER_MESSAGE_TYPE				// Arge is Keyer_Type
};

enum Keyer_State {
	KEYER_STATE_DISCONNECTED,		// Keyer is disconnected
	KEYER_STATE_INITIAL,			// Connected, initial message has been sent
	KEYER_STATE_OPERATE				// Keyer can accept characters, speed etc.
};

enum Keyer_Model {
	KEYER_MODEL_TEN_DOLLAR,			// Ten dollar keyer
	KEYER_MODEL_WK1,				// Winkey 1
	KEYER_MODEL_WK2					// Winkey 2 or 3
};

// This must agree with the order of the radio buttons in the setup dialog
// Also must agree with the ten dollar keyer!
enum Keyer_Type {
	KEYER_TYPE_UNKNOWN = -1,
	KEYER_TYPE_IAMBIC_A,
	KEYER_TYPE_IAMBIC_B,
	KEYER_TYPE_ULTIMATIC,
	KEYER_TYPE_DIT,
	KEYER_TYPE_DAH
};
static const Keyer_Type KEYER_TYPE_DEFAULT = KEYER_TYPE_IAMBIC_B;

static const UINT MIN_SPEED = 5;
static const UINT MAX_SPEED = 50;

static const UINT MIN_FREQUENCY = 300;
static const UINT MAX_FREQUENCY = 1200;
static const UINT FREQUENCY_STEP = 10;

static const UINT MIN_VOLUME = 0;
static const UINT MAX_VOLUME = 60;

class CKeyer
{
public:
	CKeyer();
	~CKeyer();

	void set_port(int port);
	void send(const unsigned char *data, int length);
	void start(HWND hwnd);

	void set_frequency(int frequency);
	void set_volume(int volume);
	void set_speed(int speed);
	void set_farnsworth(int farnsworth);
	void set_type(Keyer_Type type);

	bool get_paddle() { return m_paddle; }
	Keyer_Model get_model() { return m_keyer_model; }

private:
	int m_port;
	HANDLE m_serial_handle;
	HANDLE m_event_handle;
	OVERLAPPED m_serial_overlapped;
	Keyer_State m_keyer_state;
	Keyer_Model m_keyer_model;
	Keyer_Type m_keyer_type;
	int m_timeout;

	unsigned char m_parse_buf[PARSE_BUF_SIZE];
	int m_parse_buf_len;
	bool m_use_headphones;
	int m_speaker_volume;
	int m_headphones_volume;
	bool m_paddle;
	bool m_keyer_type_sent;

	HWND m_hwnd;
	bool m_exiting;

	void disconnect();
	bool send_internal(const unsigned char *data, int length);
	bool receive();
	void parse(unsigned char c);
	void set_type_internal(Keyer_Type type);
	DWORD connect();
	void thread_loop();


	static UINT trampoline(LPVOID param);
};

